[v6,06/17,gdbserver/generic] Convert tdesc's expedite_regs to a string vector

Message ID 20230913101815.178154-7-luis.machado@arm.com
State New
Headers
Series SME support for AArch64 gdb/gdbserver on Linux |

Checks

Context Check Description
linaro-tcwg-bot/tcwg_gdb_build--master-aarch64 success Testing passed
linaro-tcwg-bot/tcwg_gdb_build--master-arm success Testing passed
linaro-tcwg-bot/tcwg_gdb_check--master-arm success Testing passed
linaro-tcwg-bot/tcwg_gdb_check--master-aarch64 success Testing passed

Commit Message

Luis Machado Sept. 13, 2023, 10:18 a.m. UTC
  Right now the list of expedited registers is stored as an array of char *,
with a nullptr element at the end to signal its last element.

Convert expedite_regs to a std::vector of std::string so it is easier to
manage the elements and the storage is handled automatically.

Eventually we might want to convert all the target functions so they pass a
std::vector of std::string as well. Or maybe expose an interface that target can
use to add expedited registers on-by-one depending on the target description
discovery needs, as opposed to just a static list of char *.
---
 gdbserver/remote-utils.cc | 13 +++++--------
 gdbserver/tdesc.cc        | 23 ++++++++++++++---------
 gdbserver/tdesc.h         |  4 ++--
 3 files changed, 21 insertions(+), 19 deletions(-)
  

Comments

Tom Tromey Sept. 13, 2023, 3:28 p.m. UTC | #1
>>>>> "Luis" == Luis Machado via Gdb-patches <gdb-patches@sourceware.org> writes:

Luis> Right now the list of expedited registers is stored as an array of char *,
Luis> with a nullptr element at the end to signal its last element.

Luis> Convert expedite_regs to a std::vector of std::string so it is easier to
Luis> manage the elements and the storage is handled automatically.

Luis> Eventually we might want to convert all the target functions so they pass a
Luis> std::vector of std::string as well. Or maybe expose an interface that target can
Luis> use to add expedited registers on-by-one depending on the target description
Luis> discovery needs, as opposed to just a static list of char *.

Looks good to me.
Approved-By: Tom Tromey <tom@tromey.com>

Tom
  

Patch

diff --git a/gdbserver/remote-utils.cc b/gdbserver/remote-utils.cc
index 722915d30f2..fb5c38d4522 100644
--- a/gdbserver/remote-utils.cc
+++ b/gdbserver/remote-utils.cc
@@ -1068,7 +1068,6 @@  prepare_resume_reply (char *buf, ptid_t ptid, const target_waitstatus &status)
     case TARGET_WAITKIND_SYSCALL_ENTRY:
     case TARGET_WAITKIND_SYSCALL_RETURN:
       {
-	const char **regp;
 	struct regcache *regcache;
 	char *buf_start = buf;
 
@@ -1155,8 +1154,6 @@  prepare_resume_reply (char *buf, ptid_t ptid, const target_waitstatus &status)
 
 	switch_to_thread (the_target, ptid);
 
-	regp = current_target_desc ()->expedite_regs;
-
 	regcache = get_thread_regcache (current_thread, 1);
 
 	if (the_target->stopped_by_watchpoint ())
@@ -1188,11 +1185,11 @@  prepare_resume_reply (char *buf, ptid_t ptid, const target_waitstatus &status)
 	    buf += strlen (buf);
 	  }
 
-	while (*regp)
-	  {
-	    buf = outreg (regcache, find_regno (regcache->tdesc, *regp), buf);
-	    regp ++;
-	  }
+	/* Handle the expedited registers.  */
+	for (const std::string &expedited_reg :
+	     current_target_desc ()->expedite_regs)
+	  buf = outreg (regcache, find_regno (regcache->tdesc,
+					      expedited_reg.c_str ()), buf);
 	*buf = '\0';
 
 	/* Formerly, if the debugger had not used any thread features
diff --git a/gdbserver/tdesc.cc b/gdbserver/tdesc.cc
index 2c7257c458f..072b88481b2 100644
--- a/gdbserver/tdesc.cc
+++ b/gdbserver/tdesc.cc
@@ -32,14 +32,13 @@  bool target_desc::operator== (const target_desc &other) const
   if (reg_defs != other.reg_defs)
     return false;
 
-  /* Compare expedite_regs.  */
-  int i = 0;
-  for (; expedite_regs[i] != NULL; i++)
-    {
-      if (strcmp (expedite_regs[i], other.expedite_regs[i]) != 0)
-	return false;
-    }
-  if (other.expedite_regs[i] != NULL)
+  /* Compare the two vectors of expedited registers.  They will only match
+     if the following conditions are met:
+
+     - Both vectors have the same number of elements.
+     - Both vectors contain the same elements.
+     - The elements of both vectors appear in the same order.  */
+  if (expedite_regs != other.expedite_regs)
     return false;
 
   return true;
@@ -89,7 +88,13 @@  init_target_desc (struct target_desc *tdesc,
   gdb_assert (2 * tdesc->registers_size + 32 <= PBUFSIZ);
 
 #ifndef IN_PROCESS_AGENT
-  tdesc->expedite_regs = expedite_regs;
+  /* Drop the contents of the previous vector, if any.  */
+  tdesc->expedite_regs.clear ();
+
+  /* Initialize the vector with new expedite registers contents.  */
+  int expedite_count = 0;
+  while (expedite_regs[expedite_count] != nullptr)
+    tdesc->expedite_regs.push_back (expedite_regs[expedite_count++]);
 #endif
 }
 
diff --git a/gdbserver/tdesc.h b/gdbserver/tdesc.h
index 7fe7d0d8eb3..8a29fc6fe6d 100644
--- a/gdbserver/tdesc.h
+++ b/gdbserver/tdesc.h
@@ -40,9 +40,9 @@  struct target_desc final : tdesc_element
   std::vector<tdesc_feature_up> features;
 
 #ifndef IN_PROCESS_AGENT
   /* An array of register names.  These are the "expedite" registers:
      registers whose values are sent along with stop replies.  */
-  const char **expedite_regs = NULL;
+  std::vector<std::string> expedite_regs;
 
   /* Defines what to return when looking for the "target.xml" file in
      response to qXfer:features:read.  Its contents can either be
@@ -40,9 +40,9 @@  struct target_desc final : tdesc_element
 
 #ifndef IN_PROCESS_AGENT
-  /* An array of register names.  These are the "expedite" registers:
+  /* A vector of register names.  These are the "expedite" registers:
      registers whose values are sent along with stop replies.  */